@@ -19,7 +19,10 @@ from unique_toolkit._common.endpoint_requestor import (
     RequestorType,
     build_requestor,
 )
-from unique_toolkit._common.pydantic_helpers import create_union_model
+from unique_toolkit._common.pydantic_helpers import (
+    create_complement_model,
+    create_union_model,
+)
 from unique_toolkit._common.string_utilities import (
     dict_to_markdown_table,
     extract_dicts_from_string,
@@ -78,24 +81,67 @@ class HumanVerificationManagerForApiCalling(
             ]
         ],
         requestor_type: RequestorType = RequestorType.REQUESTS,
+        environment_payload_params: BaseModel | None = None,
+        modifiable_payload_params_model: type[BaseModel] | None = None,
         **kwargs: dict[str, Any],
     ):
+        """
+        Manages human verification for api calling.
+
+        Args:
+            logger: The logger to use for logging.
+            operation: The operation to use for the api calling.
+            requestor_type: The requestor type to use for the api calling.
+            environment_payload_params: The environment payload params to use for the api calling.
+                If None, the modifiable params model will be the operation payload model.
+                This can be useful for parameters in the payload that should not be modified by the user.
+            modifiable_payload_params_model: The modifiable payload params model to use for the api calling.
+                If None, a complement model will be created using the operation payload model
+                and the environment payload params.
+                If provided, it will be used instead of the complement model.
+                This is necessary if the modifiable params model is required
+                to use custom validators or serializers.
+            **kwargs: Additional keyword arguments to pass to the requestor.
+        """
         self._logger = logger
         self._operation = operation
-
+        self._environment_payload_params = environment_payload_params
         # Create internal models for this manager instance
 
-        class ConcreteApiCall(BaseModel):
+        if self._environment_payload_params is None:
+            self._modifiable_payload_params_model = self._operation.payload_model()
+        else:
+            if modifiable_payload_params_model is None:
+                self._modifiable_payload_params_model = create_complement_model(
+                    model_type_a=self._operation.payload_model(),
+                    model_type_b=type(self._environment_payload_params),
+                )
+            else:
+                # This is necessary if the modifiable params model is required
+                # to use custom validators or serializers.
+                self._modifiable_payload_params_model = modifiable_payload_params_model
+
+        if self._environment_payload_params is not None:
+            combined_keys = set(
+                self._modifiable_payload_params_model.model_fields.keys()
+            ) | set(type(self._environment_payload_params).model_fields.keys())
+            payload_keys = set(self._operation.payload_model().model_fields.keys())
+            if not payload_keys.issubset(combined_keys):
+                raise ValueError(
+                    "The modifiable params model + the environment parameters do not have all the keys of the operation payload model."
+                )
+
+        class VerificationModel(BaseModel):
             confirmation: HumanConfirmation
-            payload: self._operation.payload_model()  # type: ignore
+            modifiable_params: self._modifiable_payload_params_model  # type: ignore
 
-        self._api_call_model = ConcreteApiCall
+        self._verification_model = VerificationModel
 
+        self._requestor_type = requestor_type
         self._combined_params_model = create_union_model(
             model_type_a=self._operation.path_params_model(),
             model_type_b=self._operation.payload_model(),
         )
-        self._requestor_type = requestor_type
         self._requestor = build_requestor(
             requestor_type=requestor_type,
             operation_type=operation,
@@ -104,7 +150,10 @@ class HumanVerificationManagerForApiCalling(
         )
 
     def detect_api_calls_from_user_message(
-        self, *, last_assistant_message: ChatMessage, user_message: str
+        self,
+        *,
+        last_assistant_message: ChatMessage,
+        user_message: str,
     ) -> PayloadType | None:
         user_message_dicts = extract_dicts_from_string(user_message)
         if len(user_message_dicts) == 0:
@@ -114,13 +163,20 @@ class HumanVerificationManagerForApiCalling(
         for user_message_dict in user_message_dicts:
             try:
                 # Convert dict to payload model first, then create payload
-                api_call = self._api_call_model.model_validate(
+                verfication_data = self._verification_model.model_validate(
                     user_message_dict, by_alias=True, by_name=True
                 )
                 if self._verify_human_verification(
-                    api_call.confirmation, last_assistant_message
+                    verfication_data.confirmation, last_assistant_message
                 ):
-                    return api_call.payload
+                    payload_dict = verfication_data.modifiable_params.model_dump()
+                    if self._environment_payload_params is not None:
+                        payload_dict.update(
+                            self._environment_payload_params.model_dump()
+                        )
+
+                    return self._operation.payload_model().model_validate(payload_dict)
+
             except Exception as e:
                 self._logger.error(f"Error detecting api calls from user message: {e}")
 
@@ -141,11 +197,27 @@ class HumanVerificationManagerForApiCalling(
         return confirmation.payload_hash in last_assistant_message.content
 
     def _create_next_user_message(self, payload: PayloadType) -> str:
-        api_call = self._api_call_model(
-            payload=payload,
+        # Extract only the modifiable fields from the payload
+        payload_dict = payload.model_dump()
+        if self._environment_payload_params is not None:
+            # Remove environment params from payload to avoid validation errors
+            environment_fields = set(
+                type(self._environment_payload_params).model_fields.keys()
+            )
+            modifiable_dict = {
+                k: v for k, v in payload_dict.items() if k not in environment_fields
+            }
+        else:
+            modifiable_dict = payload_dict
+
+        modifiable_params = self._modifiable_payload_params_model.model_validate(
+            modifiable_dict
+        )
+        api_call = self._verification_model(
+            modifiable_params=modifiable_params,
             confirmation=HumanConfirmation(
                 payload_hash=hashlib.sha256(
-                    payload.model_dump_json().encode()
+                    modifiable_params.model_dump_json().encode()
                 ).hexdigest(),
                 time_stamp=datetime.now(),
             ),
@@ -168,6 +240,14 @@ class HumanVerificationManagerForApiCalling(
         path_params: PathParamsType,
         payload: PayloadType,
     ) -> ResponseType:
+        """
+        Call the api with the given path params, payload and secured payload params.
+
+        The `secured payload params` are params that are enforced by the application.
+        It should generally be not possible for the user to adapt those but here we
+        ensure that the application has the last word.
+
+        """
         params = path_params.model_dump()
         params.update(payload.model_dump())
 
@@ -200,9 +280,9 @@ if __name__ == "__main__":
     class CombinedParams(GetUserPathParams, GetUserRequestBody):
         pass
 
-    UserEndpoint = build_api_operation(
+    UserApiOperation = build_api_operation(
         method=EndpointMethods.GET,
-        url_template=Template("https://api.example.com/users/{user_id}"),
+        path_template=Template("/users/{user_id}"),
         path_params_constructor=GetUserPathParams,
         payload_constructor=GetUserRequestBody,
         response_model_type=UserResponse,
@@ -210,15 +290,15 @@ if __name__ == "__main__":
 
     human_verification_manager = HumanVerificationManagerForApiCalling(
         logger=logging.getLogger(__name__),
-        operation=UserEndpoint,
+        operation=UserApiOperation,
         requestor_type=RequestorType.FAKE,
         return_value={"id": 100, "name": "John Doe"},
     )
 
     payload = GetUserRequestBody(include_profile=True)
 
-    api_call = human_verification_manager._api_call_model(
-        payload=payload,
+    api_call = human_verification_manager._verification_model(
+        modifiable_params=payload,
         confirmation=HumanConfirmation(
             payload_hash=hashlib.sha256(payload.model_dump_json().encode()).hexdigest(),
             time_stamp=datetime.now(),
