@@ -9,6 +9,7 @@ class SubAgentResponseDisplayMode(StrEnum):
     HIDDEN = "hidden"
     DETAILS_OPEN = "details_open"
     DETAILS_CLOSED = "details_closed"
+    PLAIN = "plain"
 
 
 class SubAgentDisplayConfig(BaseModel):
@@ -22,3 +23,11 @@ class SubAgentDisplayConfig(BaseModel):
         default=True,
         description="If set, sub agent responses will be removed from the history on subsequent calls to the assistant.",
     )
+    add_quote_border: bool = Field(
+        default=True,
+        description="If set, a quote border is added to the left of the sub agent response.",
+    )
+    add_block_border: bool = Field(
+        default=False,
+        description="If set, a block border is added around the sub agent response.",
+    )
