@@ -1,8 +1,18 @@
 import logging
-from typing import Any
+from typing import Any, Sequence
 
 import unique_sdk
+from openai.types.chat import ChatCompletionToolChoiceOptionParam
 from openai.types.chat.chat_completion_message_param import ChatCompletionMessageParam
+from openai.types.responses import (
+    ResponseIncludable,
+    ResponseInputItemParam,
+    ResponseOutputItem,
+    ResponseTextConfigParam,
+    ToolParam,
+    response_create_params,
+)
+from openai.types.shared_params import Metadata, Reasoning
 from typing_extensions import deprecated
 
 from unique_toolkit._common.utils.files import is_file_content, is_image_content
@@ -37,6 +47,10 @@ from unique_toolkit.chat.functions import (
     update_message_log,
     update_message_log_async,
 )
+from unique_toolkit.chat.responses_api import (
+    stream_responses_with_references,
+    stream_responses_with_references_async,
+)
 from unique_toolkit.chat.schemas import (
     ChatMessage,
     ChatMessageAssessment,
@@ -70,11 +84,13 @@ from unique_toolkit.language_model.infos import (
     LanguageModelName,
 )
 from unique_toolkit.language_model.schemas import (
+    LanguageModelMessageOptions,
     LanguageModelMessages,
     LanguageModelResponse,
     LanguageModelStreamResponse,
     LanguageModelTool,
     LanguageModelToolDescription,
+    ResponsesLanguageModelStreamResponse,
 )
 
 logger = logging.getLogger(f"toolkit.{DOMAIN_NAME}.{__name__}")
@@ -1292,8 +1308,9 @@ class ChatService(ChatServiceDeprecated):
         debug_info: dict = {},
         temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
         timeout: int = DEFAULT_COMPLETE_TIMEOUT,
-        tools: list[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tools: Sequence[LanguageModelTool | LanguageModelToolDescription] | None = None,
         start_text: str | None = None,
+        tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         other_options: dict | None = None,
     ) -> LanguageModelStreamResponse:
         return self.complete_with_references(
@@ -1305,6 +1322,7 @@ class ChatService(ChatServiceDeprecated):
             timeout=timeout,
             tools=tools,
             start_text=start_text,
+            tool_choice=tool_choice,
             other_options=other_options,
         )
 
@@ -1316,8 +1334,9 @@ class ChatService(ChatServiceDeprecated):
         debug_info: dict | None = None,
         temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
         timeout: int = DEFAULT_COMPLETE_TIMEOUT,
-        tools: list[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tools: Sequence[LanguageModelTool | LanguageModelToolDescription] | None = None,
         start_text: str | None = None,
+        tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         other_options: dict | None = None,
     ) -> LanguageModelStreamResponse:
         """Streams a completion in the chat session synchronously."""
@@ -1336,6 +1355,7 @@ class ChatService(ChatServiceDeprecated):
             timeout=timeout,
             tools=tools,
             start_text=start_text,
+            tool_choice=tool_choice,
             other_options=other_options,
         )
 
@@ -1347,8 +1367,9 @@ class ChatService(ChatServiceDeprecated):
         debug_info: dict | None = None,
         temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
         timeout: int = DEFAULT_COMPLETE_TIMEOUT,
-        tools: list[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tools: Sequence[LanguageModelTool | LanguageModelToolDescription] | None = None,
         start_text: str | None = None,
+        tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         other_options: dict | None = None,
     ) -> LanguageModelResponse:
         response = self.complete_with_references(
@@ -1360,6 +1381,7 @@ class ChatService(ChatServiceDeprecated):
             timeout=timeout,
             tools=tools,
             start_text=start_text,
+            tool_choice=tool_choice,
             other_options=other_options,
         )
 
@@ -1374,8 +1396,9 @@ class ChatService(ChatServiceDeprecated):
         debug_info: dict | None = None,
         temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
         timeout: int = DEFAULT_COMPLETE_TIMEOUT,
-        tools: list[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tools: Sequence[LanguageModelTool | LanguageModelToolDescription] | None = None,
         start_text: str | None = None,
+        tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         other_options: dict | None = None,
     ) -> LanguageModelStreamResponse:
         """Stream a completion in the chat session asynchronously."""
@@ -1388,6 +1411,7 @@ class ChatService(ChatServiceDeprecated):
             timeout=timeout,
             tools=tools,
             start_text=start_text,
+            tool_choice=tool_choice,
             other_options=other_options,
         )
 
@@ -1399,7 +1423,8 @@ class ChatService(ChatServiceDeprecated):
         debug_info: dict | None = None,
         temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
         timeout: int = DEFAULT_COMPLETE_TIMEOUT,
-        tools: list[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tools: Sequence[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         start_text: str | None = None,
         other_options: dict | None = None,
     ) -> LanguageModelStreamResponse:
@@ -1418,6 +1443,7 @@ class ChatService(ChatServiceDeprecated):
             timeout=timeout,
             tools=tools,
             start_text=start_text,
+            tool_choice=tool_choice,
             other_options=other_options,
         )
 
@@ -1429,8 +1455,9 @@ class ChatService(ChatServiceDeprecated):
         debug_info: dict | None = None,
         temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
         timeout: int = DEFAULT_COMPLETE_TIMEOUT,
-        tools: list[LanguageModelTool | LanguageModelToolDescription] | None = None,
+        tools: Sequence[LanguageModelTool | LanguageModelToolDescription] | None = None,
         start_text: str | None = None,
+        tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         other_options: dict | None = None,
     ) -> LanguageModelResponse:
         response = self.complete_with_references_async(
@@ -1442,11 +1469,116 @@ class ChatService(ChatServiceDeprecated):
             timeout=timeout,
             tools=tools,
             start_text=start_text,
+            tool_choice=tool_choice,
             other_options=other_options,
         )
 
         return LanguageModelResponse.from_stream_response(await response)
 
+    def complete_responses_with_references(
+        self,
+        *,
+        model_name: LanguageModelName | str,
+        messages: str
+        | LanguageModelMessages
+        | Sequence[
+            ResponseInputItemParam
+            | LanguageModelMessageOptions
+            | ResponseOutputItem  # History is automatically convertible
+        ],
+        content_chunks: list[ContentChunk] | None = None,
+        tools: Sequence[LanguageModelToolDescription | ToolParam] | None = None,
+        temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
+        debug_info: dict | None = None,
+        start_text: str | None = None,
+        include: list[ResponseIncludable] | None = None,
+        instructions: str | None = None,
+        max_output_tokens: int | None = None,
+        metadata: Metadata | None = None,
+        parallel_tool_calls: bool | None = None,
+        text: ResponseTextConfigParam | None = None,
+        tool_choice: response_create_params.ToolChoice | None = None,
+        top_p: float | None = None,
+        reasoning: Reasoning | None = None,
+        other_options: dict | None = None,
+    ) -> ResponsesLanguageModelStreamResponse:
+        return stream_responses_with_references(
+            company_id=self._company_id,
+            user_id=self._user_id,
+            assistant_message_id=self._assistant_message_id,
+            user_message_id=self._user_message_id,
+            chat_id=self._chat_id,
+            assistant_id=self._assistant_id,
+            model_name=model_name,
+            messages=messages,
+            content_chunks=content_chunks,
+            tools=tools,
+            temperature=temperature,
+            debug_info=debug_info,
+            start_text=start_text,
+            include=include,
+            instructions=instructions,
+            max_output_tokens=max_output_tokens,
+            metadata=metadata,
+            parallel_tool_calls=parallel_tool_calls,
+            text=text,
+            tool_choice=tool_choice,
+            top_p=top_p,
+            reasoning=reasoning,
+            other_options=other_options,
+        )
+
+    async def complete_responses_with_references_async(
+        self,
+        *,
+        model_name: LanguageModelName | str,
+        messages: str
+        | LanguageModelMessages
+        | Sequence[
+            ResponseInputItemParam | LanguageModelMessageOptions | ResponseOutputItem
+        ],
+        content_chunks: list[ContentChunk] | None = None,
+        tools: Sequence[LanguageModelToolDescription | ToolParam] | None = None,
+        temperature: float = DEFAULT_COMPLETE_TEMPERATURE,
+        debug_info: dict | None = None,
+        start_text: str | None = None,
+        include: list[ResponseIncludable] | None = None,
+        instructions: str | None = None,
+        max_output_tokens: int | None = None,
+        metadata: Metadata | None = None,
+        parallel_tool_calls: bool | None = None,
+        text: ResponseTextConfigParam | None = None,
+        tool_choice: response_create_params.ToolChoice | None = None,
+        top_p: float | None = None,
+        reasoning: Reasoning | None = None,
+        other_options: dict | None = None,
+    ) -> ResponsesLanguageModelStreamResponse:
+        return await stream_responses_with_references_async(
+            company_id=self._company_id,
+            user_id=self._user_id,
+            assistant_message_id=self._assistant_message_id,
+            user_message_id=self._user_message_id,
+            chat_id=self._chat_id,
+            assistant_id=self._assistant_id,
+            model_name=model_name,
+            messages=messages,
+            content_chunks=content_chunks,
+            tools=tools,
+            temperature=temperature,
+            debug_info=debug_info,
+            start_text=start_text,
+            include=include,
+            instructions=instructions,
+            max_output_tokens=max_output_tokens,
+            metadata=metadata,
+            parallel_tool_calls=parallel_tool_calls,
+            text=text,
+            tool_choice=tool_choice,
+            top_p=top_p,
+            reasoning=reasoning,
+            other_options=other_options,
+        )
+
     # Chat Content Methods
     ############################################################################
 
